/************************************************************************/
/*               (C) Fujitsu Semiconductor Europe GmbH (FSEU)           */ 
/*                                                                      */
/* The following software deliverable is intended for and must only be  */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the user's obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/* FSEU does not warrant that the deliverables do not infringe any      */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* To the maximum extent permitted by applicable law FSEU disclaims all */
/* warranties, whether express or implied, in particular, but not       */
/* limited to, warranties of merchantability and fitness for a          */
/* particular purpose for which the deliverable is not designated.      */
/*                                                                      */
/* To the maximum extent permitted by applicable law, FSEU's liability  */
/* is restricted to intentional misconduct and gross negligence.        */
/* FSEU is not liable for consequential damages.                        */
/*                                                                      */
/* (V1.5)                                                               */
/************************************************************************/
   
#include "mb9bf506n.h"
#include "system_mb9bf50x.h"

const char DEC7SEG[16]={0xC0, 0xF9, 0xA4, 0xB0, 0x99, 0x92, 0x82, 0xF8, 0x80, 0x90, 0x88, 0x83, 0xC6, 0xA1, 0x86, 0x8E};

volatile unsigned int adc_value;

void delay(unsigned int d)
{
  while (d>0)
  {
    d = d - 1;  
  }
}

int main(void)
{
  // 7-Segment Display
  
  FM3_GPIO->PFR1  = 0x0000;  // use Port1 as GPIO
  FM3_GPIO->PDOR1 = 0xBF00;  // set P1F:P18 to 10111111 (7-Segm. Display '-')
  FM3_GPIO->DDR1  = 0xFF00;  // set P1F:P18 as output

  FM3_GPIO->PFR3  = 0x0000;  // use Port3 as GPIO
  FM3_GPIO->PDOR3 = 0xBF00;  // set P39:P32 to 10111111 (7-Segm. Display '-')
  FM3_GPIO->DDR3  = 0xFF00;  // set P39:P32 as output

  // ADC Init
  
  FM3_GPIO->ADE   = 0x0001;  // Select analogue pin -> AN00

  FM3_ADC0->SCIS3 = 0x00; // Scan channel select AN31-AN24
  FM3_ADC0->SCIS2 = 0x00; // Scan channel select AN23-AN16
  FM3_ADC0->SCIS1 = 0x00; // Scan channel select AN15-AN08
  FM3_ADC0->SCIS0 = 0x01; // Scan channel select AN07-AN00 -> AN00

  FM3_ADC0->ADST1 = 0x2F; // Sampling Time 1
  FM3_ADC0->ADST0 = 0x2F; // Sampling Time 0

  FM3_ADC0->ADSS3 = 0x00; // Sampling Time Select AN31-AN24
  FM3_ADC0->ADSS2 = 0x00; // Sampling Time Select AN23-AN16
  FM3_ADC0->ADSS1 = 0x00; // Sampling Time Select AN15-AN08
  FM3_ADC0->ADSS0 = 0x00; // Sampling Time Select AN07-AN00 -> Use Samplin Time 0 for AN0

  FM3_ADC0->ADCT  = 0x00; // Comparison Time = 14 x (ADCT + 2) / HCLK

  FM3_ADC0->ADCEN = 0x01; // Enable ADC    

  while (3 != FM3_ADC0->ADCEN); // wait until ADC operation is enabled

  FM3_ADC0->ADSR  = 0x00; // Stop ADC, Places conversion result on the MSB side. 

  FM3_ADC0->CMPCR = 0x00; // No comparsion
  FM3_ADC0->CMPD  = 0x00; // No comparsion value

  FM3_ADC0->SFNS  = 0x00; // Set Fifo Stage Count Interrupt
  FM3_ADC0->ADCR  = 0x00; // Disable ADC interrupts   
  
  FM3_ADC0->SCCR = 0x11; // FIFO clear, start ADC Single Conversion   

  while (1)
  {
    if ( 0 == ((FM3_ADC0->SCFDL) & 0x1000)) // Is valid data available?
    {
      adc_value = (FM3_ADC0->SCFDH) >> 8;            // Take the upper 8bits of the 12bit ADC result
      FM3_GPIO->PDOR1 = DEC7SEG[adc_value>>4]<<8;    // and show it on the display
      FM3_GPIO->PDOR3 = DEC7SEG[adc_value&0x0F]<<8;
      bFM3_GPIO_PDOR1_PF = 1;                        // Do not show decimal point
      delay(8000000);

      adc_value = (50*adc_value)/255;                       // Calculate the voltage 0..5V
      FM3_GPIO->PDOR1 = DEC7SEG[(adc_value/10)&0x0F]<<8;
      FM3_GPIO->PDOR3 = DEC7SEG[(adc_value%10)&0x0F]<<8;    // and show it on the display
      bFM3_GPIO_PDOR1_PF = 0;                               // Show decimal point
      delay(8000000);

      bFM3_ADC0_SCCR_SSTR = 1; // Start ADC Single Conversion
    }
  }
}
