/************************************************************************/
/*               (C) Fujitsu Semiconductor Europe GmbH (FSEU)           */
/*                                                                      */
/* The following software deliverable is intended for and must only be  */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the user's obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/* FSEU does not warrant that the deliverables do not infringe any      */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* To the maximum extent permitted by applicable law FSEU disclaims all */
/* warranties, whether express or implied, in particular, but not       */
/* limited to, warranties of merchantability and fitness for a          */
/* particular purpose for which the deliverable is not designated.      */
/*                                                                      */
/* To the maximum extent permitted by applicable law, FSEU's liability  */
/* is restricted to intentional misconduct and gross negligence.        */
/* FSEU is not liable for consequential damages.                        */
/*                                                                      */
/* (V1.5)                                                               */
/************************************************************************/

#include "mb9bf506n.h"
#include "system_mb9bf50x.h"
#include "type.h"

// LED number pattern array (0 - 9)
const unsigned short LEDPATTERN[10] = 
  {0xC000, 0xF900, 0xA400, 0xB000, 0x9900, 0x9200, 0x8200, 0xF800, 0x8000, 0x9000};

unsigned char count = 0;          // counter for LED displays
signed char count_direction = 1;  // count direction 

// ---------------- wait loop -----------------
void wait (int a)
{
  while(a--);
}

// -------------- main program ----------------
int32_t main(void)
{
  FM3_GPIO->ADE  = 0x00FF; // No Analog Inputs
  
  FM3_GPIO->DDR1 = 0xFF00;  // P18-P1F: LED-SEG1 output
  FM3_GPIO->DDR3 = 0xFF00;  // P38-P3F: LED-SEG2 output
    
  FM3_GPIO->PFR1 = 0x0000;  // P10-P1F: LED-SEG1 GPIO
  FM3_GPIO->PFR3 = 0x0000;  // P30-P3F: LED-SEG2 GPIO
  
  FM3_GPIO->DDR5 = 0x0000;  // P18-P1F: SW2-INT0, SW3-INT1 input
    
  FM3_GPIO->PFR5 = 0x0000;  // P10-P1F: SW2-INT0, SW3-INT1 GPIO

  while(1)
  {
    FM3_GPIO->PDOR1 = LEDPATTERN[count / 10];                   // set tens' place 
    FM3_GPIO->PDOR3 = LEDPATTERN[count - ((count / 10) * 10)];  // set ones' place
    
    wait (1000000);                     // wait some time ....
    
    if (!(0 == (FM3_GPIO->PDIR5 & 0x0003))) // both key buttons not pressed?
    {
      if (0 == (FM3_GPIO->PDIR5 & 0x0001))  // SW2-INT0 pressed?
      {
        count_direction = -1;           // count direction upwards
      }
  
      if (0 == (FM3_GPIO->PDIR5 & 0x0002))  // SW3-INT1 pressed?
      {
        count_direction = 1;            // count direction downwards
      }
    }
        
    if ((1 == count_direction ) && (count < 99))  // still count upwards?
    {
      count++;                          // count upwards, if < 99
    }
    
    if ((-1 == count_direction ) && (count != 0)) // still count downwards?
    {
      count--;                          // count downwards, if != 0
    }   
    
  }
}

