/************************************************************************/
/*   (C) FUJITSU SEMICONDUCTOR LIMITED AND ITS SUBSIDIARIES (FSLGROUP)  */
/*                                                                      */
/* The following software deliverable is subject the readme text        */
/* included in the same directory or parent directory of this software  */
/* and the use agreement as found on FSLGROUPs Website                 */
/* www.fujitsu.com/global/services/microelectronics/product/micom/      */ 
/* regarding this software.                                             */ 
/* If you do not agree with such terms then DO NOT use this software    */
/* Some highlights of the use agreement are as follows:                 */
/* This software is intended for and must only be                       */
/* used for reference and in an evaluation laboratory environment.      */
/* It is provided on an as-is basis without charge and is subject to    */
/* alterations.                                                         */
/* It is the users obligation to fully test the software in its        */
/* environment and to ensure proper functionality, qualification and    */
/* compliance with component specifications.                            */
/*                                                                      */
/* In the event the software deliverable includes the use of open       */
/* source components, the provisions of the governing open source       */
/* license agreement shall apply with respect to such software          */
/* deliverable.                                                         */
/*                                                                      */
/* FSLGROUP does not warrant that the deliverables do not infringe any  */
/* third party intellectual property right (IPR). In the event that     */
/* the deliverables infringe a third party IPR it is the sole           */
/* responsibility of the customer to obtain necessary licenses to       */
/* continue the usage of the deliverable.                               */
/*                                                                      */
/* FSLGROUP disclaims all warranties, whether express or implied,       */
/* in particular, but not limited to, warranties of merchantability     */
/* and fitness for a particular purpose                                 */
/*                                                                      */
/* To the maximum extent permitted by applicable law,                   */
/* FSLGROUPs liability is restricted to intentional misconduct         */
/* and gross negligence.                                                */
/*                                                                      */
/* FSLGROUP is not liable for consequential damages.                    */                             
/*                                                                      */
/* (V2.0)                                                               */
/************************************************************************/

#include "mcu.h"

const unsigned short LEDPATTERN1[16]={0x2008, 0xF408, 0x4808, 0xC008, 0x9408, 0x8208, 0x0208, 0xF008, 0x0008, 0x8008, 0x1008, 0x0608, 0x2A08, 0x4408, 0x0A08, 0x1A08};
const unsigned short LEDPATTERN2[16]={0x0008, 0x007C, 0x0011, 0x0030, 0x0064, 0x0022, 0x0020, 0x0078, 0x0000, 0x0020, 0x0080, 0x0006, 0x000B, 0x0014, 0x0003, 0x0043};

volatile unsigned int adc_value;

void delay(unsigned int d)
{
  while (d>0)
  {
    d = d - 1;  
  }
}

int main(void)
{
  // 7-Segment Display
  FM3_GPIO->ADE   = 0x0000;
  FM3_GPIO->PFR5 &=0xFF87;   // select CPIO function
  FM3_GPIO->PDOR5 &= 0xFF87;  // set pins to turn off leds
  FM3_GPIO->DDR5 |= 0x0078;   // make led pins outputs
  
  FM3_GPIO->PFR3 &=0x01F0;   // select CPIO function
  FM3_GPIO->PDOR3 &= 0x01F0;  // set pins to turn off leds
  FM3_GPIO->DDR3 |= 0xFE0F;   // make led pins outputs

  // ADC Init
  
  FM3_GPIO->ADE   |= 0x1000;  // Select analogue pin -> AN24

  FM3_ADC0->SCIS3 = 0x01; // Scan channel select AN31-AN24 -> AN24
  FM3_ADC0->SCIS2 = 0x00; // Scan channel select AN23-AN16
  FM3_ADC0->SCIS1 = 0x00; // Scan channel select AN15-AN08
  FM3_ADC0->SCIS0 = 0x00; // Scan channel select AN07-AN00 

  FM3_ADC0->ADST1 = 0x2F; // Sampling Time 1
  FM3_ADC0->ADST0 = 0x2F; // Sampling Time 0

  FM3_ADC0->ADSS3 = 0x00; // Sampling Time Select AN31-AN24
  FM3_ADC0->ADSS2 = 0x00; // Sampling Time Select AN23-AN16
  FM3_ADC0->ADSS1 = 0x00; // Sampling Time Select AN15-AN08
  FM3_ADC0->ADSS0 = 0x00; // Sampling Time Select AN07-AN00 -> Use Samplin Time 0 for AN0

  FM3_ADC0->ADCT  = 0x00; // Comparison Time = 14 x (ADCT + 2) / HCLK

  FM3_ADC0->ADCEN = 0x01; // Enable ADC    

  while (3 != FM3_ADC0->ADCEN); // wait until ADC operation is enabled

  FM3_ADC0->ADSR  = 0x00; // Stop ADC, Places conversion result on the MSB side. 

  FM3_ADC0->CMPCR = 0x00; // No comparsion
  FM3_ADC0->CMPD  = 0x00; // No comparsion value

  FM3_ADC0->SFNS  = 0x00; // Set Fifo Stage Count Interrupt
  FM3_ADC0->ADCR  = 0x00; // Disable ADC interrupts   
  
  FM3_ADC0->SCCR = 0x11; // FIFO clear, start ADC Single Conversion   

  while (1)
  {
    if ( 0 == ((FM3_ADC0->SCFDL) & 0x1000)) // Is valid data available?
    {
      adc_value = (FM3_ADC0->SCFDH) >> 8;            // Take the upper 8bits of the 12bit ADC result
      FM3_GPIO->PDOR5 = 0x78 &LEDPATTERN2[adc_value&0x0F];    // and show it on the display
      FM3_GPIO->PDOR3 = 0x07 &LEDPATTERN2[adc_value&0x0F];
      FM3_GPIO->PDOR3 |= LEDPATTERN1[adc_value>>4];
      FM3_GPIO->PDOR3 |= 0x0008;
        
      delay(8000000);
      delay(8000000);

      adc_value = (50*adc_value)/255;                       // Calculate the voltage 0..5V
      FM3_GPIO->PDOR5 = 0x78 &LEDPATTERN2[(adc_value%10)&0x0F];    // and show it on the display
      FM3_GPIO->PDOR3 = 0x07 &LEDPATTERN2[(adc_value%10)&0x0F]; 
      FM3_GPIO->PDOR3 |= LEDPATTERN1[(adc_value/10)&0x0F];
      FM3_GPIO->PDOR3 &=0xFFF7;
      delay(8000000);
      delay(8000000);

      bFM3_ADC0_SCCR_SSTR = 1; // Start ADC Single Conversion
    }
  }
}
